/******************************************************************************
 *
 * Copyright (c) 1999-2004 PalmSource, Inc. All rights reserved.
 *
 * File: AppMain.c
 *
 *****************************************************************************/

#include <PalmOS.h>

#include "AppResources.h"


/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'hfsT'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01


/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/

#define SCALERES(x)    (x) //*2)

#define LOGTOP 12 //0
#define LOGLEFT 0
#define LOGWIDTH	160
#define LOGHEIGHT	132//160

#define SCROLL	11	

#include <stdint.h>
#include <stdarg.h>
#include <HostControl.h>

static RectangleType r = { LOGLEFT, LOGTOP,(SCALERES(LOGWIDTH)),(SCALERES(LOGHEIGHT)) };
void scroll(){
    RectangleType /* r, */ empty;
/*
    r.topLeft.x = LOGLEFT;
    r.topLeft.y =	LOGTOP;
    r.extent.x = (SCALERES(LOGWIDTH));
    r.extent.y = (SCALERES(LOGHEIGHT));
*/
    WinScrollRectangle(&r, winUp, SCROLL, &empty);
    WinEraseRectangle(&empty, 0);
}
void print(char *format, ...)
{
    static char buf[2048];
    va_list args;
    va_start(args, format);
    StrVPrintF(buf, format, args);
    va_end(args);
	scroll();
    WinDrawChars(buf, StrLen(buf), LOGLEFT, SCALERES(LOGHEIGHT));
}

//#define HostIsSelectorImplemented(x) (1)

void testHostFS(){
Err err; 

	print("Starting...");
	if (HostIsSelectorImplemented(hostSelectorGetHostVersion)){
		Int32 hVer=HostGetHostVersion();
		print("HostControl version %d.%d implemented.",sysGetROMVerMajor(hVer),sysGetROMVerMinor(hVer));
	}
	if (HostIsSelectorImplemented(hostSelectorGetHostPlatform)){
		HostIDType hID=HostGetHostID();
		HostPlatformType hPlat=HostGetHostPlatform();
		print("Host ID %ld platform %ld.",hID,hPlat);
	}
	if (HostIsSelectorImplemented(hostSelectorSlotMax)){
		long i,maxSlots=HostSlotMax();
		print("Host slots %ld",maxSlots);
		for (i=0;i<maxSlots;i++){
			long hasCard=HostSlotHasCard(i);
			char *root=HostSlotRoot(i);
			if (root==NULL) root="";
			print("Slot#%ld has card %ld, rootdir=",i,hasCard);
			print("%s",(StrLen(root)<256) ? root : "");
		}
	}
	if (HostIsSelectorImplemented(hostSelectorTraceInit)){
		HostTraceInit();
		print("testing HostTraceOutput Hello World");
		HostTraceOutputTL(sysErrorClass,"Hello %s","world");
		HostTraceClose();
	}
	if (HostIsSelectorImplemented(hostSelectorFOpen)){
		HostFILEType*f=HostFOpen("hostfstestfile","w");
		print("testing HostFOpen");
		if (f!=NULL){
			print("testing HostFWrite/FClose");
			HostFWrite("Hello", 5, 1, f);
			HostFClose(f);
		}
	}
	print("Finished.");
}

static Boolean MainFormDoButton(UInt16 command)
{
	Boolean handled = false;
	FormType * pForm;

	switch (command) {
		case MainGoButton:
			testHostFS();			
			handled = true;
			break;

	}
	
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormType * pForm;

	switch (command) {
		case MainOptionsAboutStarterApp:
			pForm = FrmInitForm(AboutForm);
			FrmDoDialog(pForm);					// Display the About Box.
			FrmDeleteForm(pForm);
			handled = true;
			break;

	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  pEvent  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventType* pEvent)
{
	Boolean 	handled = false;
	FormType* 	pForm;

	switch (pEvent->eType) {
		case menuEvent:
			return MainFormDoCommand(pEvent->data.menu.itemID);

		case ctlSelectEvent:
			return MainFormDoButton(pEvent->data.ctlSelect.controlID);

		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			FrmDrawForm(pForm);
			handled = true;
			break;
			
		default:
			break;
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventType* pEvent)
{
	UInt16 		formId;
	FormType* 	pForm;
	Boolean		handled = false;

	if (pEvent->eType == frmLoadEvent) {
		// Load the form resource.
		formId = pEvent->data.frmLoad.formID;
		
		pForm = FrmInitForm(formId);
		FrmSetActiveForm(pForm);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId) {
			case MainForm:
				FrmSetEventHandler(pForm, MainFormHandleEvent);
				break;

			default:
				break;
		}
		handled = true;
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value errNone if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	FrmGotoForm(MainForm);
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	// Close all the open forms.
	FrmCloseAllForms();
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	Err			error;
	EventType	event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (SysHandleEvent(&event))
			continue;
			
		if (MenuHandleEvent(0, &event, &error))
			continue;
			
		if (AppHandleEvent(&event))
			continue;

		FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY: 
 *
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error = errNone;

	switch (cmd) {
		case sysAppLaunchCmdNormalLaunch:
			if ((error = AppStart()) == 0) {			
				AppEventLoop();
				AppStop();
			}
			break;

		default:
			break;
	}
	
	return error;
}


